#!/usr/dist/bin/wishx -f

#
# tzclock.tcl
# Copyright (C) 1995-1998 by John Heidemann
# This program is released under the Gnu Public License, version 2.
# Comments to <johnh@isi.edu>.
#

proc timeshift_to_timeshiftsecs {ts} {
	if [regexp -- {^([+-]?)([0-9]+):([0-9][0-9])$} $ts x sign hours minutes] {
		set tss [expr "${sign}1" * ( $hours * 60 + $minutes ) * 60]
	} else {
		set tss 0
		puts "tzclock: regexp on ts fails: $ts"
	}
	return $tss
}

proc tzclock_new_timeshift {new_ts} {
	global rdb
	set rdb(tzclock.timeshift) $new_ts
	set rdb(tzclock.timeshiftsecs) [timeshift_to_timeshiftsecs $rdb(tzclock.timeshift)]
}

proc tzclock_update {} {
	global rdb monitor_module
	if {$monitor_module(tzclock)} {
		set seconds [expr [clock seconds] - $rdb(tzclock.timeshiftsecs)]
		if { $rdb(tzclock.format) == "" } {
#			set day [string trimleft [clock format $seconds -format %d] 0]
			set hour [string trimleft [clock format $seconds -format %I] 0]
#			set tm [string trimleft [clock format $seconds -format "\[$hour:%M\] %a %b $day"] "0"]
			set tm [string trimleft [clock format $seconds -format "\[$hour:%M\]"] "0"]
		} else {
			set tm [clock format $seconds -format $rdb(tzclock.format)]
			regsub -all -- { 0} $tm { } tm
		}
		.tzclock config -text $tm
		after [expr 1000*(160-1[clock format $seconds -format %S])] tzclock_update
	}
}


proc tzclock_ok_dialog {w} {
	tzclock_new_timeshift [string trim [$w.tzdiff get 0.0 end]]
	destroy $w
}

proc tzclock_cancel_dialog {w} {
	destroy $w
	tzclock_stop
}

proc tzclock_make_dialog {} {
	global tk_version
	set w ".tzclockDialog"
	if [winfo exists $w] {
		return
	}
	toplevel $w
	set wn "Monitor--TzClock"
	wm title $w $wn
	wm iconname $w $wn
	wm transient $w .

	set padValue 4
	set halfPad 2
	set doublePad 8

	frame $w.top
	frame $w.bottom

	label $w.icon -bitmap question
	label $w.query -text "Time-zone difference (h:mm):" -anchor w
	text $w.tzdiff \
		-relief sunken -bd 2 \
		-pady $halfPad -padx $halfPad \
		-height 1 -width 20
	$w.tzdiff mark set insert 0.0
	$w.tzdiff insert end "-3:00"
	button $w.cancel -text Cancel -padx $doublePad \
		-command "tzclock_cancel_dialog $w"
	button $w.ok -text OK -padx $doublePad \
		-command "tzclock_ok_dialog $w"

	pack $w.query $w.tzdiff -in $w.top -side top -anchor w
	pack $w.ok $w.cancel -in $w.bottom \
		-pady $padValue -padx $padValue -side right

	pack $w.icon -side left -anchor nw \
		-padx $padValue -pady $padValue
	pack $w.top -side top \
		-padx $padValue -pady $padValue
	pack $w.bottom -side bottom -anchor e \
		-padx $padValue -pady $padValue

	# bindings
	set target $w.tzdiff
	if {$tk_version < 4.0} { bind $w <Any-Enter> "focus $target" }
	bind $target <Any-KeyPress-Return> "tzclock_ok_dialog $w"
	bind $target <Any-KeyPress-KP_Enter> "tzclock_ok_dialog $w"
	bind $target <Any-KeyPress-Escape> "tzclock_cancel_dialog $w"
}

proc tzclock_start {} {
	global rdb
	set_rdb_default tzclock.format ""
	set_rdb_default tzclock.row 1
	set_rdb_default tzclock.timeshift -3:00
	tzclock_new_timeshift $rdb(tzclock.timeshift)
	mk_rows $rdb(tzclock.row)
	label .tzclock -text "tzclock" -font $rdb(font) -fg $rdb(fg) -bg $rdb(bg)
	pack .tzclock -side $rdb(side) -in ".row$rdb(tzclock.row)"
	tzclock_update
	tzclock_make_dialog
}



proc tzclock_stop {} {
	destroy .tzclock
}
